@@ -37,14 +37,12 @@ new(Size) ->
     #none{size = Size}.
 
 update(#none{size = Size, reservoir = Reservoir, n = N} = Sample, Value)
-  when N < Size ->
+  when N =:= Size ->
     ets:insert(Reservoir, {N, Value}),
-    Sample#none{n = folsom_utils:get_ets_size(Reservoir) + 1};
+    Sample#none{n = 1};
 update(#none{reservoir = Reservoir, n = N} = Sample, Value) ->
-    Oldest = ets:first(Reservoir),
-    ets:delete(Reservoir, Oldest),
     ets:insert(Reservoir, {N, Value}),
-    Sample#none{n = folsom_utils:get_ets_size(Reservoir) + 1}.
+    Sample#none{n = N  + 1}.
 
 get_values(#none{reservoir = Reservoir}) ->
     {_, Values} = lists:unzip(ets:tab2list(Reservoir)),
